Package: MACSQuantifyR
Type: Package
Title: Fast treatment of MACSQuantify FACS data
Date: 08/10/2019
Version: 1.25.0
Author: Raphaël Bonnet [aut, cre], Marielle Nebout [dtc],Giulia Biondani [dtc], Jean-François Peyron[aut,ths], Inserm [fnd]
Maintainer: Raphaël Bonnet <raphael.bonnet@univ-cotedazur.fr>
Description: Automatically process the metadata of MACSQuantify FACS sorter. 
    It runs multiple modules: i) imports of raw file and graphical selection of duplicates in well plate, 
    ii) computes statistics on data and iii) can compute combination index.
Imports: readxl, graphics, tools, utils, grDevices, ggplot2, ggrepel,
        methods, stats, latticeExtra, lattice, rmarkdown, png, grid,
        gridExtra, prettydoc, rvest, xml2
biocViews: DataImport, Preprocessing, Normalization, FlowCytometry,
        DataRepresentation, GUI
License: Artistic-2.0
Encoding: UTF-8
RoxygenNote: 7.1.0
Suggests: knitr, testthat, R.utils, spelling
VignetteBuilder: knitr
Language: en-US
git_url: https://git.bioconductor.org/packages/MACSQuantifyR
git_branch: devel
git_last_commit: 1dffa00
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 21:45:19 UTC; biocbuild
